SET TERM ^ ;
ALTER PROCEDURE PROC_ETAT_LONGEVITE (A_CLE_DOSSIER INTEGER,
LIMIT_ON_DATE INTEGER,
LIMIT_ON_SOSA INTEGER,
YEAR_FROM INTEGER,
YEAR_TO INTEGER,
LINTERVAL INTEGER)
RETURNS (INTERVAL_START INTEGER,
INTERVAL_END INTEGER,
NB_HOMME INTEGER,
MOYENNE_AGE_HOMME decimal(15,2),
ECART_TYPE_AGE_HOMME decimal(15,2),
NB_FEMME INTEGER,
MOYENNE_AGE_FEMME decimal(15,2),
ECART_TYPE_AGE_FEMME decimal(15,2))
AS 
declare variable ok_suspend integer;
declare variable age double precision;
declare variable annee_de_deces integer;
declare variable sexe integer;
declare variable cumul_carre_homme double precision;
declare variable cumul_carre_femme double precision;
declare variable cumul_age_homme double precision;
declare variable cumul_age_femme double precision;
begin
  ok_suspend=0;
  for select (d.ev_ind_datecode-n.ev_ind_datecode)/365.25
    ,i.annee_deces
    ,i.sexe
  from individu i
  inner join evenements_ind n on n.ev_ind_kle_fiche=i.cle_fiche and n.ev_ind_type='BIRT'
  inner join evenements_ind d on d.ev_ind_kle_fiche=i.cle_fiche and d.ev_ind_type='DEAT'
  where i.kle_dossier=:a_cle_dossier
  and i.sexe>0
  and n.ev_ind_datecode is not null
  and d.ev_ind_datecode is not null
  and (:limit_on_sosa=0 or num_sosa>0)
  and (:limit_on_date=0 or i.annee_deces between :year_from and :year_to)
  order by i.annee_deces
  into :age
      ,:annee_de_deces
      ,:sexe
  do
  begin
    if (ok_suspend=0) then --c'est le premier enregistrement
    begin
      --initialisation des intervals et autorisation dernier suspend
      interval_start=annee_de_deces-mod(annee_de_deces,LINTERVAL);
      interval_end=interval_start+LINTERVAL-1;
      nb_homme=0;
      cumul_age_homme=0;
      cumul_carre_homme=0;
      nb_femme=0;
      cumul_age_femme=0;
      cumul_carre_femme=0;
      ok_suspend=1;
    end
    else
    begin
      if (annee_de_deces>interval_end) then --l'année est dans un LINTERVAL suivant
      begin
        --on expédie le record
        if (nb_homme>0) then
        begin
          moyenne_age_homme=cumul_age_homme/nb_homme;
          ecart_type_age_homme=cumul_carre_homme/nb_homme-moyenne_age_homme*moyenne_age_homme;
          if (ecart_type_age_homme>0) then
            ecart_type_age_homme=sqrt(ecart_type_age_homme);
          else
            ecart_type_age_homme=0;
        end
        else
        begin
          moyenne_age_homme=0;
          ecart_type_age_homme=0;
        end
        if (nb_femme>0) then
        begin
          moyenne_age_femme=cumul_age_femme/nb_femme;
          ecart_type_age_femme=cumul_carre_femme/nb_femme-moyenne_age_femme*moyenne_age_femme;
          if (ecart_type_age_femme>0) then
            ecart_type_age_femme=sqrt(ecart_type_age_femme);
          else
            ecart_type_age_femme=0;
        end
        else
        begin
          moyenne_age_femme=0;
          ecart_type_age_femme=0;
        end
        suspend;
        --changement d'LINTERVAL
        while (annee_de_deces>interval_end) do
        begin
          --initialisation du suspend
          interval_start=interval_start+LINTERVAL;
          interval_end=interval_start+LINTERVAL-1;
          nb_homme=0;
          cumul_age_homme=0;
          cumul_carre_homme=0;
          nb_femme=0;
          cumul_age_femme=0;
          cumul_carre_femme=0;
          --si l'année est encore après, on édite
          if (annee_de_deces>interval_end) then
          begin
            moyenne_age_homme=0;
            moyenne_age_femme=0;
            ecart_type_age_homme=0;
            ecart_type_age_femme=0;
            suspend;
          end
        end
      end
    end
    if (sexe=1) then
    begin
      nb_homme=nb_homme+1;
      cumul_age_homme=cumul_age_homme+age;
      cumul_carre_homme=cumul_carre_homme+age*age;
    end
    else
    begin
      nb_femme=nb_femme+1;
      cumul_age_femme=cumul_age_femme+age;
      cumul_carre_femme=cumul_carre_femme+age*age;
    end
  end
  if (ok_suspend=1) then
  begin
    if (nb_homme>0) then
    begin
      moyenne_age_homme=cumul_age_homme/nb_homme;
      ecart_type_age_homme=cumul_carre_homme/nb_homme-moyenne_age_homme*moyenne_age_homme;
      if (ecart_type_age_homme>0) then
        ecart_type_age_homme=sqrt(ecart_type_age_homme);
      else
        ecart_type_age_homme=0;
    end
    else
    begin
      moyenne_age_homme=0;
      ecart_type_age_homme=0;
    end
    if (nb_femme>0) then
    begin
      moyenne_age_femme=cumul_age_femme/nb_femme;
      ecart_type_age_femme=cumul_carre_femme/nb_femme-moyenne_age_femme*moyenne_age_femme;
      if (ecart_type_age_femme>0) then
        ecart_type_age_femme=sqrt(ecart_type_age_femme);
      else
        ecart_type_age_femme=0;
    end
    else
    begin
      moyenne_age_femme=0;
      ecart_type_age_femme=0;
    end
    suspend;
  end
end ^
ALTER PROCEDURE PROC_ETAT_DENOMB_ASCEND (
    A_CLE_FICHE integer )
RETURNS (
    NIVEAU integer,
    TOTAL_INDI integer,
    TOTAL_INDI_DISTINCT integer,
    TOTAL_INDI_THEORIQUE decimal(15,2),
    CUMUL_INDI integer,
    CUMUL_INDI_DISTINCT integer,
    CUMUL_INDI_THEORIQUE decimal(15,2),
    POURCENT_IMPLEXE decimal(15,2))

AS
begin
  delete from tq_ascendance;
  insert into tq_ascendance (tq_niveau,tq_cle_fiche)
           values (1,:a_cle_fiche);
  niveau=1;
  total_indi=1;
  total_indi_theorique=1;
  cumul_indi_theorique=0;
  delete from tq_consang;
  select 100*consanguinite
    from proc_consang(:a_cle_fiche,0,10)
    into :pourcent_implexe;
  delete from tq_consang;
  while (total_indi>0) do
  begin
    insert into tq_ascendance (tq_niveau,tq_cle_fiche)
      select :niveau+1,i.cle_pere
      from tq_ascendance t
      inner join individu i on i.cle_fiche=t.tq_cle_fiche
      where t.tq_niveau=:niveau;
    insert into tq_ascendance (tq_niveau,tq_cle_fiche)
      select :niveau+1,i.cle_mere
      from tq_ascendance t
      inner join individu i on i.cle_fiche=t.tq_cle_fiche
      where t.tq_niveau=:niveau;
    select count(tq_cle_fiche),count(distinct tq_cle_fiche)
    from tq_ascendance where tq_niveau=:niveau+1
    into :total_indi,:total_indi_distinct;
    if (total_indi>0) then
    begin
      select count(tq_cle_fiche)-1,count(distinct tq_cle_fiche)-1
      from tq_ascendance
      into :cumul_indi,:cumul_indi_distinct;
      total_indi_theorique=total_indi_theorique*2;
      cumul_indi_theorique=cumul_indi_theorique+total_indi_theorique;
      if (niveau>1) then
        pourcent_implexe=100*cast(cumul_indi-cumul_indi_distinct as double precision)/cumul_indi;
      niveau=niveau+1;
      suspend;
      if (niveau>100) then total_indi=0;
    end
  end
  delete from tq_ascendance;
end
^
ALTER PROCEDURE PROC_ETAT_DENOMB_DESCEND (
    A_CLE_FICHE integer )
RETURNS (
    NIVEAU integer,
    TOTAL_INDI integer,
    TOTAL_INDI_DISTINCT integer,
    CUMUL_INDI integer,
    CUMUL_INDI_DISTINCT integer,
    POURCENT_IMPLEXE decimal(15,2))

AS
begin
  delete from tq_ascendance;
  insert into tq_ascendance (tq_niveau,tq_cle_fiche)
         values (1,:a_cle_fiche);
  niveau=1;
  total_indi=1;
  while (total_indi>0) do
  begin
    insert into tq_ascendance (tq_niveau,tq_cle_fiche)
      select :niveau+1, i.cle_fiche
      from tq_ascendance t
      inner join individu i on i.cle_pere=t.tq_cle_fiche or i.cle_mere=t.tq_cle_fiche
      where t.tq_niveau=:niveau;
    select count(tq_cle_fiche),count(distinct tq_cle_fiche)
    from tq_ascendance where tq_niveau=:niveau+1
    into :total_indi,:total_indi_distinct;
    if (total_indi>0) then
    begin
      select count(tq_cle_fiche)-1,count(distinct tq_cle_fiche)-1
      from tq_ascendance
      into :cumul_indi,:cumul_indi_distinct;
      pourcent_implexe=100*cast(cumul_indi-cumul_indi_distinct as double precision)/cumul_indi;
      niveau=niveau+1;
      suspend;
      if (niveau>100) then total_indi=0;
    end
  end
  delete from tq_ascendance;
end
^
SET TERM ; ^
comment on trigger T_BI_INDIVIDU is 'Trigger modifié par MG
Dernière modification 14/12/2023';

UPDATE T_VERSION_BASE SET VER_VERSION='5.183';
COMMIT WORK;
